#include <stdio.h>
#include <malloc.h>

#include <kallisti/stdtypes.h>
#include <kallisti/fs.h>
#include <kallisti/libk.h>

#include <kallisti/abi/fs.h>
#include <kallisti/abi/thread.h>
#include <kallisti/abi/ta.h>
#include <kallisti/abi/maple.h>
#include <kallisti/abi/video.h>

#include "vmu_icon.h"
#include "filelist.h"
#include "image.h"

#define CD_ONLY
#define RNDC 1013904223
#define RNDM 1164525

abi_fs_t *fs;
abi_ta_t *ta;
abi_thread_t *thread;
abi_maple_t *maple;
abi_video_t *video;

unsigned int font_tex;
uint32 bkg_tex;
unsigned long seed = 123;
int nxtimg=0;

filelist_t list;
filelist_t *selected;
filelist_t *displist;

void font_init(void)
{
    unsigned char font[4096];
    int i,x,y,c;
    unsigned int fd;
    unsigned short *tex;

    fd = fs->open("/rd/8x16.bin", O_RDONLY);
    fs->read(fd, font, 4096);
    fs->close(fd);

    font_tex = ta->txr_allocate(256*128*2);
    tex = ta->txr_map(font_tex);
    
    c = 0;
    for(y = 0; y < 128 ; y+=16)
        for(x = 0; x < 256 ; x+=8) {
            for(i = 0; i < 16; i++) {
                tex[x + (y+i) * 256 + 0] = 0xffff * ((font[c+i] & 0x80)>>7);
                tex[x + (y+i) * 256 + 1] = 0xffff * ((font[c+i] & 0x40)>>6);
                tex[x + (y+i) * 256 + 2] = 0xffff * ((font[c+i] & 0x20)>>5);
                tex[x + (y+i) * 256 + 3] = 0xffff * ((font[c+i] & 0x10)>>4);
                tex[x + (y+i) * 256 + 4] = 0xffff * ((font[c+i] & 0x08)>>3);
                tex[x + (y+i) * 256 + 5] = 0xffff * ((font[c+i] & 0x04)>>2);
                tex[x + (y+i) * 256 + 6] = 0xffff * ((font[c+i] & 0x02)>>1);
                tex[x + (y+i) * 256 + 7] = 0xffff * (font[c+i] & 0x01);
            }
            c+=16;
        }
}


void draw_color_rect(int x, int y, int w, int h, float a, float r, float g, float b, float z, int trans)
{
    vertex_oc_t vert;
    poly_hdr_t poly;

    if (!trans)
	ta->poly_hdr_col(&poly, TA_OPAQUE);
    else
	ta->poly_hdr_col(&poly, TA_TRANSLUCENT);
    ta->commit_poly_hdr(&poly);

    vert.flags = TA_VERTEX_NORMAL;
    vert.x = x;
    vert.y = y+h-1;
    vert.z = z;
    vert.a = a;
    vert.r = r;
    vert.g = g;
    vert.b = b;
    ta->commit_vertex(&vert, sizeof(vert));

    vert.y = y;
    ta->commit_vertex(&vert, sizeof(vert));

    vert.x = x+w-1;
    vert.y = y+h-1;
    ta->commit_vertex(&vert, sizeof(vert));
	
    vert.flags = TA_VERTEX_EOL;
    vert.y = y;
    ta->commit_vertex(&vert, sizeof(vert));
}

void draw_char(float x1, float y1, float z1, float a, float r, float g, float b, int c, float xs, float ys) {
    vertex_ot_t     vert;
    int             ix, iy;
    float           u1, v1, u2, v2;
    
    ix = (c % 32) * 8;
    iy = (c / 32) * 16;
    u1 = (float)(ix) / 256.0f;
    v1 = (float)(iy) / 256.0f;
    u2 = (ix+8.0f) / 256.0f;
    v2 = (iy+16.0f) / 256.0f;
    
    vert.flags = TA_VERTEX_NORMAL;
    vert.x = x1;
    vert.y = y1 + 16.0f * ys;
    vert.z = z1;
    vert.u = u1;
    vert.v = v2;
    vert.dummy1 = vert.dummy2 = 0;
    vert.a = a;
    vert.r = r;
    vert.g = g;
    vert.b = b;
    vert.oa = vert.or = vert.og = vert.ob = 0.0f;
    ta->commit_vertex(&vert, sizeof(vert));
    
    vert.x = x1;
    vert.y = y1;
    vert.u = u1;
    vert.v = v1;
    ta->commit_vertex(&vert, sizeof(vert));
    
    vert.x = x1 + 8.0f * xs;
    vert.y = y1 + 16.0f * ys;
    vert.u = u2;
    vert.v = v2;
    ta->commit_vertex(&vert, sizeof(vert));
    
    vert.flags = TA_VERTEX_EOL;
    vert.x = x1 + 8.0f * xs;
    vert.y = y1;
    vert.u = u2;
    vert.v = v1;
    ta->commit_vertex(&vert, sizeof(vert));
}

void draw_string(float x, float y, float z, float a, float r, float g, float b, char *str, float xs, float ys) {
  poly_hdr_t poly;
  ta->poly_hdr_txr(&poly, TA_TRANSLUCENT, TA_ARGB1555, 256, 256,
                   font_tex, TA_BILINEAR_FILTER);
  ta->commit_poly_hdr(&poly);
  
  while (*str) {
    draw_char(x, y, z, a, r, g, b, *str++, xs, ys);
        x+=8*xs;
  }
}

void draw_string_center(float y, float z, float a, float r, float g, float b, char *str, float xs, float ys)
{
    draw_string((640.0f - strlen(str)*8*xs)/2, y, z, a, r, g, b, str, xs, ys);
}

void show_list(filelist_t *list, filelist_t *selected)
{
    int y = 30;
    filelist_t *item = list;
    static float alpha = 1.0f, dalpha = -0.01f;

    while((item)&&(y<460)) {

	if (item == selected) {
	    
	    draw_string(18, y, 6.0f, 1.0f, 0.0f, 0.0f, 1.0f, item->dispname, 2.0f, 2.0f);
	    draw_color_rect(8, y, 616, 32, alpha, 1.0f, 1.0f, 1.0f, 5.2f, 1);
	} else
	    draw_string(18, y, 6.0f, 1.0f, 1.0f, 1.0f, 1.0f, item->dispname, 2.0f, 2.0f);

	item = item->next;
	
	y += 40;
	
    }

    alpha += dalpha;
    if (alpha <= 0.2f)
	dalpha = -dalpha;
    if (alpha >= 1.0f)
	dalpha = -dalpha;
}

void check_scroll()
{
    int i;
    filelist_t *item;

    if (selected == displist->prev) {
	displist = selected;
	return;
    }

    item = displist;

    i = 0;
    while((item != selected)&&(item)) {
	item = item->next;
	i++;
    }

    if (i==10)
	displist = displist->next;

}

void draw_one_frame(void)
{
    poly_hdr_t poly;

    ta->poly_hdr_col(&poly, TA_TRANSLUCENT);

    ta->begin_render();

    ta->commit_eol();

    show_list(displist, selected);

    ta->commit_poly_hdr(&poly);

    ta->commit_eol();
    ta->finish_frame();
}

void back_one_dir(char *path)
{
    int i;

    i = strlen(path) - 2;

#ifdef CD_ONLY
    if (i<7)
	return;
#else
    if (i<0)
	return;
#endif

    while(i>=0) {
	if (path[i] == '/') {
	    path[i+1] = 0;
	    break;
	} 
	i--;
    }

    if (strlen(path) < 1)
	strcpy(path, "/");
}

void vmu_icon_init() 
{
    int x, y, xi, xb;
    uint8 bitmap[48*32/8];
    uint8 addr;

    addr = maple->first_vmu();
    if (!addr) return;

    memset(bitmap, 0, 48*32/8);
    for (y=0; y<32; y++)
	for (x=0; x<48; x++) {
	    xi = x / 8;
	    xb = 0x80 >> (x % 8);
	    if (vmu_icon[(31-y)*48+(47-x)] == '.')
		bitmap[y*(48/8)+xi] |= xb;
	}
    maple->vmu_draw_lcd(addr, bitmap);
}

void show_image(char *filename){


	float width, height;
    cont_cond_t cond;
    float zoom=1.0f, x1=-320.0f,y1=240.0f, oldx=0.0f, oldy=480.0f,oldmidx=0.5f, oldmidy=0.5f;
	int x,bifilt;
	image img;
    ta->txr_release_all();
	JPEG_to_image(filename, &img);
	font_init(); 

	nxtimg=0;
    while(1) {
		
        if (maple->cont_get_cond(maple->first_controller(), &cond) < 0)
            break;
        
        if (!(cond.buttons & CONT_START)){
			nxtimg=0;
            break;
		}
            
        if( !(cond.buttons & CONT_B)){
			nxtimg=0;
        	break;
		}
        	
        if( !(cond.buttons & CONT_Y)){
			nxtimg=0;
        	break;
		}
        	
        if( !(cond.buttons & CONT_A)){
			nxtimg=1;
        	break;
		}
        if( !(cond.buttons & CONT_X)){
        	zoom=1.0f;
        	oldmidx=0.5f;
        	oldmidy=0.5f;
        }
        	      	
        if( !(cond.buttons & CONT_DPAD_UP))
        	oldmidy+=0.001f;
        
        if( !(cond.buttons & CONT_DPAD_DOWN))
        	oldmidy-=0.001f;
        	
        if( !(cond.buttons & CONT_DPAD_LEFT))
        	oldmidx-=0.001f;
        	
        if( !(cond.buttons & CONT_DPAD_RIGHT))
        	oldmidx+=0.001f;
        if( cond.ltrig>5){
	        zoom-=cond.ltrig/10000.0f;
        	if( img.horiz_polys*64.0f*zoom < 64.0f){
        		zoom=64.0f/(img.horiz_polys*64.0f);
        	}else if(img.vert_polys*64.0f*zoom < 64.0f){
        		zoom=64.0f/(img.vert_polys*64.0f);
        	}
        }

        if( cond.rtrig>5)
        	zoom+=cond.rtrig/10000.0f;

        if( cond.joyx-128>5 || cond.joyx-128<-5)
        	oldmidx+=(cond.joyx-128)/60000.0f;
        if( cond.joyy-128>5 || cond.joyy-128<-5)
        	oldmidy-=(cond.joyy-128)/60000.0f;
        	
	ta->begin_render();

	width=img.horiz_polys*64.0f*zoom;
	height=img.vert_polys*64.0f*zoom;
	x1=-oldmidx*width;
	y1=oldmidy*height;
	draw_image(320.0f+x1,240.0f+y1, width, height, bifilt, &img, 1.0f);

	
	ta->commit_eol(); 
	draw_string(20,40, 6.0f, 1.0f, 0.4f, 0.4f, 1.0f, selected->dispname, 1.5f, 1.5f);
	{
		poly_hdr_t	hdr;
		ta->poly_hdr_col(&hdr, TA_TRANSLUCENT);
		ta->commit_poly_hdr(&hdr);
		ta->commit_eol();
	}

	ta->finish_frame();

    } 
}


void browse(){

    cont_cond_t cond;
    cont_cond_t oldcond;
    unsigned char times = 0;
    uint32 wasdown = 0;
    uint32 wasup = 0;
    char filename[512];

	strcpy(pathname, "/cd/pic/");
    build_list(&list);
    selected = &list;
    displist = &list;

    
    while(1) {
	if (maple->cont_get_cond(maple->first_controller(), &cond) < 0)
	    break;
	if (!(cond.buttons & CONT_START))
	    break;

	if (!(cond.buttons & CONT_DPAD_UP))
	    if ((oldcond.buttons & CONT_DPAD_UP)||(times == 0)) {
		if (selected->prev)
		    selected = selected->prev;
		if (times)
		    times = 0;
	    }
	if (!(cond.buttons & CONT_DPAD_DOWN))
	    if ((oldcond.buttons & CONT_DPAD_DOWN)||(times == 0)) {
		if (selected->next)
		    selected = selected->next;
		if (times)
		    times = 0;
	    }
	if (nxtimg==1){
		if (selected->next)
		    selected = selected->next;
		if (times)
		    times = 0;
		if (selected->size == -1) {
		    if (strcmp(selected->realname, ".")) {
			if (!strcmp(selected->realname, ".."))
			    back_one_dir(pathname);
			else {
			    if (selected->realname[0] != '/')
				strcat(pathname, selected->realname);
			    else
				strcat(pathname, &selected->realname[1]);
			    strcat(pathname, "/");
			}	
			destroy_list(&list);
			build_list(&list);
			selected = &list;
			displist = &list;
		    }
		} else {
		    strcpy(filename, pathname);
		    strcat(filename, selected->realname);
		    show_image(filename);
		}
	}
	if (!(cond.buttons & CONT_A))
	    if (oldcond.buttons & CONT_A) {
		if (selected->size == -1) {
		    if (strcmp(selected->realname, ".")) {
			if (!strcmp(selected->realname, ".."))
			    back_one_dir(pathname);
			else {
			    if (selected->realname[0] != '/')
				strcat(pathname, selected->realname);
			    else
				strcat(pathname, &selected->realname[1]);
			    strcat(pathname, "/");
			}	
			destroy_list(&list);
			build_list(&list);
			selected = &list;
			displist = &list;
		    }
		} else {
		    strcpy(filename, pathname);
		    strcat(filename, selected->realname);
		    show_image(filename);
		}
		}
	if (!(cond.buttons & CONT_B))
	    if (oldcond.buttons & CONT_B) {
		back_one_dir(pathname);
		destroy_list(&list);
		build_list(&list);
		selected = &list;
		displist = &list;
	    }

	times++;
	times = times % 12;
	oldcond = cond;

	check_scroll();

	draw_one_frame();
	thread->pass();

    }
    ta->txr_release_all();
    font_init();
}

int32 rand() {
  seed = seed * RNDM + RNDC;
  return seed;
}

void slideshow(){
    cont_cond_t cond;
	int32 k=0,x=0,l=0;
	float i;
    char filename[512];
 	image img;
    int change=0;
	static unsigned int dirsize=0;
    
	strcpy(pathname, "/cd/saver/");
    build_list(&list);
    selected = &list;
    displist = &list;
	
    ta->txr_release_all();
    font_init();

	dirsize=0;
	while(selected->next){
		selected = selected->next;
		dirsize++;
    }

	while(1){
		if (maple->cont_get_cond(maple->first_controller(), &cond) < 0)
		    break;
		if (!(cond.buttons & CONT_START))
	    	break;

		strcpy(pathname, "/cd/saver/");
		build_list(&list);
		selected = &list;
		displist = &list;
	    	
			
		x = rand() % (dirsize+3);
		for(l=0; l < x; l++)
		if (selected->next){
			selected = selected->next;
			change++;
		}
		check_scroll();

		if (selected->size == -1) {
		    if (strcmp(selected->realname, ".")) {
			if (!strcmp(selected->realname, ".."))
			    back_one_dir(pathname);
			else {
			    if (selected->realname[0] != '/')
				strcat(pathname, selected->realname);
			    else
				strcat(pathname, &selected->realname[1]);
			    strcat(pathname, "/");
			}	
			destroy_list(&list);
			build_list(&list);
			selected = &list;
			displist = &list;
		    }
		} else {
		    strcpy(filename, pathname);
		    strcat(filename, selected->realname);
			change=0;
		    ta->txr_release_all();
			JPEG_to_image(filename, &img);
			font_init(); 
		    for(i=0.0f; i<1.0f; i+=0.05f){
		   		if (maple->cont_get_cond(maple->first_controller(), &cond) < 0)
		    		break;
				if (!(cond.buttons & CONT_START))
	    			break;
				ta->begin_render();
		    	draw_image_full(0, &img, i);
		    	ta->commit_eol(); 
				draw_string(20,40, 6.0f, i, 0.4f, 0.4f, 1.0f, selected->dispname, 1.5f, 1.5f);
				{
					poly_hdr_t	hdr;
					ta->poly_hdr_col(&hdr, TA_TRANSLUCENT);
					ta->commit_poly_hdr(&hdr);
					ta->commit_eol();
				}

				ta->finish_frame();
		    }
		    for(i=0.0f; i<400.0f; i+=1.0f){
		    	if (maple->cont_get_cond(maple->first_controller(), &cond) < 0)
		    		break;
				if (!(cond.buttons & CONT_START))
	    			break;
	    		if( !(cond.buttons & CONT_A)){
        			ta->txr_release_all();
        			show_image(filename);
        		    ta->txr_release_all();
        		    JPEG_to_image(filename, &img);
    				font_init();
    			}
				ta->begin_render();
		    	draw_image_full(0, &img, 1.0f);
		    	ta->commit_eol(); 
				draw_string(20,40, 6.0f, 1.0f, 0.4f, 0.4f, 1.0f, selected->dispname, 1.5f, 1.5f);
				{
					poly_hdr_t	hdr;
					ta->poly_hdr_col(&hdr, TA_TRANSLUCENT);
					ta->commit_poly_hdr(&hdr);
					ta->commit_eol();
				}

				ta->finish_frame();
		    }
		    for(i=1.0f; i>= -0.1f; i-=0.05f){
		    	if (maple->cont_get_cond(maple->first_controller(), &cond) < 0)
		    		break;
				if (!(cond.buttons & CONT_START))
	    			break;
				ta->begin_render();
		    	draw_image_full(0, &img, i);
		    	ta->commit_eol(); 
				draw_string(20,40, 6.0f, i, 0.4f, 0.4f, 1.0f, selected->dispname, 1.5f, 1.5f);
				{
					poly_hdr_t	hdr;
					ta->poly_hdr_col(&hdr, TA_TRANSLUCENT);
					ta->commit_poly_hdr(&hdr);
					ta->commit_eol();
				}


				ta->finish_frame();
		    }
		    
		}
		check_scroll();
	}
	nxtimg=0;
	ta->txr_release_all();
    font_init();
}

void display_info(){
	cont_cond_t cond;

	ta->txr_release_all();
	font_init();

	while(1){
		if (maple->cont_get_cond(maple->first_controller(), &cond) < 0){
		    break;
		}
		if (!(cond.buttons & CONT_START))
	    			break;
		if (!(cond.buttons & CONT_X))
	    			break;
		if (!(cond.buttons & CONT_A))
	    			break;
		if (!(cond.buttons & CONT_B))
	    			break;
		if (!(cond.buttons & CONT_DPAD_UP))
	    			break;
		if (!(cond.buttons & CONT_DPAD_DOWN))
	    			break;

		ta->begin_render();

		{
			poly_hdr_t	hdr;
			ta->poly_hdr_col(&hdr, TA_OPAQUE);
			ta->commit_poly_hdr(&hdr);
			ta->commit_eol();
		}

		draw_string_center(100, 6.0f, 1.0f, 1.0f, 1.0f, 1.0f, "This program was written by Jon Reeser", 1.8f, 1.8f);
		draw_string_center(140, 6.0f, 1.0f, 1.0f, 1.0f, 1.0f, "using KallistiOS v1.0.0 by Cryptic", 1.8f, 1.8f);
		draw_string_center(180, 6.0f, 1.0f, 1.0f, 1.0f, 1.0f, "Allusions and Andrew Kieschnick's port", 1.8f, 1.8f);
		draw_string_center(220, 6.0f, 1.0f, 1.0f, 1.0f, 1.0f, "of jpeg-6b. Modified code form", 1.8f, 1.8f);
		draw_string_center(260, 6.0f, 1.0f, 1.0f, 1.0f, 1.0f, "AndrewK/NAPALM's MP3 player, from the", 1.8f, 1.8f);
		draw_string_center(300, 6.0f, 1.0f, 1.0f, 1.0f, 1.0f, "DCTONIC CD, was used for the", 1.8f, 1.8f);
		draw_string_center(340, 6.0f, 1.0f, 1.0f, 1.0f, 1.0f, "file browser and font.", 1.8f, 1.8f);

		ta->commit_eol();

		ta->finish_frame();
	}

}

void menu(){
	
	image bro;
	image slide;
	image bak;
	int cho=1;
	cont_cond_t cond;
	float bro_a=1.0f, sli_a=0.4f;
    
    ta->txr_release_all();
	
	JPEG_to_image("/rd/home2.jpg", &bak);
	font_init();
		
	while(1){

		seed++;
		ta->begin_render();
		draw_image_full(0, &bak, 1.0f);
		draw_string_center(180, 6.0f, bro_a, 1.0f, 0.4f, 0.4f, "BROWSER", 3.0f, 3.0f);
		draw_string_center(290, 6.0f, sli_a, 1.0f, 0.4f, 0.4f, "SLIDESHOW", 3.0f, 3.0f);
		draw_string_center(400, 6.0f, 1.0f, 1.0f, 0.4f, 0.4f, "Press Y for information", 1.8f, 1.8f);

		ta->commit_eol();
		{
			poly_hdr_t	hdr;
			ta->poly_hdr_col(&hdr, TA_TRANSLUCENT);
			ta->commit_poly_hdr(&hdr);
			ta->commit_eol();
		}
		
		ta->finish_frame();
		if (maple->cont_get_cond(maple->first_controller(), &cond) < 0){
		    break;
		}

		if (!(cond.buttons & CONT_A)){
			if(cho==1){
				browse();
			    ta->txr_release_all();
	
				JPEG_to_image("/rd/home2.jpg", &bak);
				font_init();
			}else if(cho==2){
				slideshow();
			    ta->txr_release_all();
	
				JPEG_to_image("/rd/home2.jpg", &bak);
				font_init();
			}
		}  
		if (!(cond.buttons & CONT_Y)){
			display_info();
			ta->txr_release_all();	
			JPEG_to_image("/rd/home2.jpg", &bak);
			font_init();
		}
		if (!(cond.buttons & CONT_DPAD_UP)){
			cho=1;
			bro_a=1.0f;
			sli_a=0.4f;
		}
		if (!(cond.buttons & CONT_DPAD_DOWN)){
			cho=2;
			bro_a=0.4f;
			sli_a=1.0f;
		}
	}
}

void change_disk(){

	image bak;
	cont_cond_t cond;
	
	JPEG_to_image("/rd/switch.jpg", &bak);
	font_init();
		
	while(1){
		seed++;
		ta->begin_render();
		draw_image_full(0, &bak, 1.0f);

		ta->commit_eol();
		{
			poly_hdr_t	hdr;
			ta->poly_hdr_col(&hdr, TA_TRANSLUCENT);
			ta->commit_poly_hdr(&hdr);
			ta->commit_eol();
		}
		
		ta->finish_frame();
		if (maple->cont_get_cond(maple->first_controller(), &cond) < 0){
		    break;
		}

		if (!(cond.buttons & CONT_A)){
			break;
		}

	}
	menu();
}

int main(int argc, char **argv) 
{
    ta = lib_open("ta");
    if (ta == NULL) { printf("Can't open TA lib\r\n"); return -1; }
    fs = lib_open("fs");
    if (fs == NULL) { printf("Can't open FS lib\r\n"); return -1; }
    maple = lib_open("maple");
    if (maple == NULL) { printf("Can't open Maple lib\r\n"); return -1; }
    thread = lib_open("thread");
    if (thread == NULL) { printf("Can't open Thread lib\r\n"); return -1; }
    video = lib_open("video");
    if (video == NULL) { return -1; }
 //////////////////////////////////////
 video->set_mode(video->check_cable(), DM_640x480, PM_RGB565);
 //////////////////////////////////////
    vmu_icon_init();

    font_init();


	change_disk();


   
    /* Force a rescan of the CD */
    {

    uint32 hnd; 
    hnd = fs->open(pathname, O_RDONLY | O_DIR);
    fs->ioctl(hnd, NULL, 0);
    fs->close(hnd);

    hnd = fs->open("/cd", O_RDONLY | O_DIR);
    fs->close(hnd);

    }
                
    destroy_list(&list);

    return 0;
}

